package com.jz.dmp.modules.service.impl;

import com.jz.agent.service.DmpDsAgentService;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.utils.JsonMapper;
import com.jz.common.utils.StringUtils;
import com.jz.common.utils.web.XmlUtils;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.modules.dao.DmpDevelopTaskDao;
import com.jz.dmp.modules.dao.DmpProjectDao;
import com.jz.dmp.modules.dao.DmpSyncingDatasourceDao;
import com.jz.dmp.modules.dao.DmpSyncingDatasourceTypeDao;
import com.jz.dmp.modules.model.*;
import com.jz.dmp.modules.service.DmpDevelopTaskService;
import com.jz.dmp.modules.service.DmpSyncingDatasourceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 任务开发(DmpDevelopTask)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-22 16:37:56
 */
@Service("dmpDevelopTaskService")
@Transactional
public class DmpDevelopTaskServiceImpl implements DmpDevelopTaskService {

    private static Logger logger = LoggerFactory.getLogger(OfflineSynchServiceImpl.class);

    @Autowired
    private DmpDevelopTaskDao dmpDevelopTaskDao;

    @Autowired
    private DmpProjectDao dmpProjectDao;

    @Autowired
    private DmpSyncingDatasourceTypeDao dmpSyncingDatasourceTypeDao;

    @Autowired
    private DmpDsAgentService dmpDsAgentServiceImp;

    @Autowired
    private DmpSyncingDatasourceService dmpSyncingDatasourceService;

    /**
     * 添加保存dmp数据(包含校验数据)
     *
     * @Author Bellamy
     * @Date 2020/12/20
     */
    @Override
    public JsonResult submitSyncing(DmpDevelopTask task) throws Exception {
        String xmlContent;
        try {
            xmlContent = convert2SyncXmlContent(task);
            System.out.println(xmlContent);
        } catch (Exception e) {
            logger.error("封装离线同步xml内容出错：" + e.getMessage(), e);
            return new JsonResult(ResultCode.PARAMS_ERROR);
        }
        return submitSyncXml(task.getProjectId(), xmlContent);
    }

    /**
     * 提交离线同步到指定projectId项目下（上传xml配置文件）
     */
    private JsonResult submitSyncXml(long projectId, String xmlContent) {
        String _type_ = XmlUtils.getPropertyValue(xmlContent, "type");
        String _fileName_ = XmlUtils.getPropertyValue(xmlContent, "name");
        String _dbName_ = XmlUtils.getPropertyValue(xmlContent, "target_database");
        String _targetFtpConnection_ = XmlUtils.getPropertyValue(xmlContent, "target_ftp_connection");
        if (StringUtils.isNull(_dbName_)) { // 2ftp类型中无target_database标签
            if (_type_.toLowerCase().indexOf("2ftp") != -1) {
                _dbName_ = _targetFtpConnection_;
            } else {
                throw new RuntimeException("离线同步配置缺失目标数据源名称！");
            }
        }

        DmpAgentDatasourceInfo ds = new DmpAgentDatasourceInfo();
        DmpProjectSystemInfo info = dmpProjectDao.queryProjectSystemInfo(projectId);
        ds.setKerberosIsenable(info.getKerberosIsenable());
        ds.setKerberosJaasConf(info.getKerberosJaasConf());
        ds.setKerberosKrb5Conf(info.getKerberosKrb5Conf());
        ds.setKerberosFqdn(info.getKerberosFqdn());
        ds.setHdfsDefaultConf(""); // info.getHdfsDefaultConf());
        ds.setHdfsConfPath(""); // info.getHdfsConfPath());
        ds.setHdfsAuthPath(info.getKerberosKeytabConf()); // info.getHdfsAuthPath());
        ds.setHdfsUserName(info.getHdfsUserName());
        ds.setHdfsSyncingPath(info.getHdfsSyncingPath());
        ds.setDefaultFs(info.getHdfsHttpPath());

        ds.setTargetDbName(_dbName_);
        ds.setTargetFileName(_fileName_ + ".xml");
        ds.setProjectId(String.valueOf(projectId));
        DmpAgentResult rst = dmpDsAgentServiceImp.submitSycingXmlConfig(ds, xmlContent);

        if (!rst.getCode().val().equals("200")) {
            return new JsonResult(rst.getCode(), rst.getMessage());
        } else {
            rst.setResult(JsonMapper.fromJsonString(rst.getMessage(), Boolean.class));
            return new JsonResult(ResultCode.SUCCESS, rst);
        }
    }

    /*
     * 生成xml
     * */
    public String convert2SyncXmlContent(DmpDevelopTask body) throws Exception {
        logger.info("################################## 拼接xml 开始 ############################################");
        StringBuilder sb = new StringBuilder();
        long treeId = body.getTreeId();
        //通过树节点id 查询任务信息
        DmpDevelopTask task = dmpDevelopTaskDao.selectTaskInfoByParam(treeId);

        Map<String, Object> map = (Map<String, Object>) JsonMapper.fromJsonString(task.getScript(), Map.class);

        Map<String, Object> scriptMap = (Map<String, Object>) map.get("scripts");
        Object content = scriptMap.get("content");
        if (null != content && StringUtils.hasLength(content.toString())) {
            // 脚本模式-XML内容
            sb.append(content);
        } else {
            // 脚本模式-json内容，解析JSON串并组装XML内容
            Map<String, Object> settingMap = (Map<String, Object>) scriptMap.get("setting");
            String _extract = (String) settingMap.get("extract");
            String _extractExpression = (String) settingMap.get("extractExpression");
            String _targetBucketCounts = (String) settingMap.get("targetBucketCounts");
            String _errorLimitRecord = (String) settingMap.get("errorLimitRecord");//错误记录数超过
            String _executorMemory = (String) settingMap.get("executorMemory");
            String _executorCores = (String) settingMap.get("executorCores");
            String _totalExecutorCores = (String) settingMap.get("totalExecutorCores");
            String _ftColumn = (String) settingMap.get("ftColumn");//分桶字段
            String _ftCount = (String) settingMap.get("ftCount");//分桶个数
            String _separateMax = (String) settingMap.get("separateMax");//分桶字段最大值
            String _separateMin = (String) settingMap.get("separateMin");//分桶字段最小值
            String _syncRate = (String) settingMap.get("syncRate");//同步速率
            String _maxConcurrency = (String) settingMap.get("maxConcurrency");//最大并发数
            String _preImportStatement = (String) settingMap.get("preImportStatement");//导入前语句
            String _postImportStatement = (String) settingMap.get("postImportStatement");//导入后语句

            //******源数据******
            Map<String, Object> readerMap = (Map<String, Object>) scriptMap.get("reader");
            String _dbConnection = (String) readerMap.get("dbConnection"); //源数据库名称
            String _fileType = (String) readerMap.get("fileType"); //文件类型
            String _sourceHdfsPath = (String) readerMap.get("sourceHdfsPath");
            String _sourceHdfsFile = (String) readerMap.get("sourceHdfsFile");
            String _sourceFtpDir = (String) readerMap.get("sourceFtpDir");
            String _sourceFtpFile = (String) readerMap.get("sourceFtpFile");
            String _sourceSkipFtpFile = (String) readerMap.get("sourceSkipFtpFile");
            String _sourceCsvDelimiter = (String) readerMap.get("sourceCsvDelimiter");
            String _sourceCsvHeader = (String) readerMap.get("sourceCsvHeader");
            String _sourceCsvCharset = (String) readerMap.get("sourceCsvCharset");
            String _sourceCsvQuote = (String) readerMap.get("sourceCsvQuote");
            String _sourceFtpLoadDate = (String) readerMap.get("sourceFtpLoadDate");
            String _registerTableName = (String) readerMap.get("registerTableName");//源数据库表名称
            String registerTableName_ = (String) readerMap.get("registerTableName");
            String _dayByDay = (String) readerMap.get("dayByDay");
            List<Map<String, Object>> _readerColumns = (List<Map<String, Object>>) readerMap.get("column"); //源数据库表字段

            //******目标数据******
            Map<String, Object> writerMap = (Map<String, Object>) scriptMap.get("writer");
            String _targetDbConnection = (String) writerMap.get("targetDbConnection"); //目标数据库
            String _targetTable = (String) writerMap.get("targetTable"); //目标数据库表名称
            String _targetFtpDir = (String) writerMap.get("targetFtpDir");
            String _targetFtpFile = (String) writerMap.get("targetFtpFile");
            String _targetCsvDelimiter = (String) writerMap.get("targetCsvDelimiter");
            String _targetCsvHeader = (String) writerMap.get("targetCsvHeader");
            String _targetCsvCharset = (String) writerMap.get("targetCsvCharset");
            String _targetInsertMergeOverwrite = (String) writerMap.get("targetInsertMergeOverwrite");  //同步策略
            List<Map<String, Object>> _writerColumns = (List<Map<String, Object>>) writerMap.get("column");//目标数据库表字段

            Integer _projectId_ = body.getProjectId(); //项目id

            //根据项目id和数据源名称 查询数据信息
            DmpSyncingDatasource sDS = this.getDmpSyncingDatasource(_projectId_, _dbConnection);//源
            DmpSyncingDatasource tDS = this.getDmpSyncingDatasource(_projectId_, _targetDbConnection);//目标

            if (null == sDS || null == tDS || null == sDS.getDatasourceType() || null == tDS.getDatasourceType())
                throw new Exception("同步数据来源或数据去向信息有误");
            //根据数据源类型ID 获取数据源类型
            DmpSyncingDatasourceType sDST = dmpSyncingDatasourceTypeDao.queryById(sDS.getDatasourceType());//源
            DmpSyncingDatasourceType tDST = dmpSyncingDatasourceTypeDao.queryById(tDS.getDatasourceType());//目标

            String _targetDbName_ = this.getTargetDBname(tDS);
            String _sourceDbName_ = this.getTargetDBname(sDS);
            String st = sDST.getDatasourceType();
            String tt = tDST.getDatasourceType();

            //获取文件类型
            String ft = this.getFileType(_fileType, _sourceHdfsFile, _sourceFtpFile);

            boolean isBigData2jdbc = ("HIVE".equals(st) || "KUDU".equals(st)) && "RDBMS".equals(tDST.getDatasourceCatecode());
            boolean isBigData2ftp = ("HIVE".equals(st) || "KUDU".equals(st)) && "FTP".equals(tt);

            if ("RDBMS".equals(sDST.getDatasourceCatecode()) && "HIVE".equals(tt)) {
                _registerTableName = sDS.getDatasourceName() + "_" + _registerTableName.substring(_registerTableName.indexOf(".") + 1);
            }

            String name_ = (StringUtils.hasLength(_targetDbName_) ? _targetDbName_ : tDS.getDatasourceName()) + "_" + _targetTable;
            String type_ = this.getEtlProcessType(sDST, tDST, _fileType);

            String[] sci = this.genSourceColumnsInfo(sDST, tDST, ft, _registerTableName, registerTableName_, _extract,
                    _extractExpression, _readerColumns, _sourceDbName_);
            String source_table_ = sci[0];
            String source_columns_ = sci[1];
            String source_sql_ = sci[2];

            String target_table_ = _targetTable;
            String[] tci = this.genTargetColumnsInfo(sDST, tDST, _targetTable, _targetBucketCounts, _writerColumns);
            String target_columns_ = tci[0];
            String target_primary_columns_ = tci[1];
            String target_partition_columns_ = tci[2];
            String target_table_creation_ = tci[3];

            String clean_rules_ = sci[3] + tci[4];
            String check_rules_ = sci[4] + tci[5];

            sb.append("<etlprocess>").append("\r\n");
            sb.append("<name>").append(name_).append("</name>").append("\r\n");
            sb.append("<type>").append(type_).append("</type>").append("\r\n");

            if ("RDBMS".equals(sDST.getDatasourceCatecode())) {
                sb.append("<source_table db_connection=\"").append(sDS.getDatasourceName()).append("\" register_table_name=\"")
                        .append(_registerTableName.substring(_registerTableName.indexOf(".") + 1)).append("\">").append(source_table_).append("</source_table>").append("\r\n");
            }
            if ("HIVE".equals(st) || "KUDU".equals(st)) {
                String sourceTableName = _registerTableName;
                if ("KUDU".equals(st)) {
                    sourceTableName = "impala::" + _sourceDbName_ + "." + _registerTableName;
                }
                sb.append("<source_table db_connection=\"").append(sDS.getDatasourceName()).append("\" register_table_name=\"")
                        .append(_registerTableName).append("\">").append(sourceTableName).append("</source_table>").append("\r\n");
            }

            if (isBigData2ftp) { // 前端考虑到源中可能包含字段信息，分开处理
                _sourceCsvDelimiter = _targetCsvDelimiter;
                _sourceCsvCharset = _targetCsvCharset;
                _sourceCsvHeader = _targetCsvHeader;
            }

            if ("csv".equals(ft) || "FTP".equals(st) || isBigData2ftp) {
                sb.append("<source_csv_delimiter>").append(_sourceCsvDelimiter).append("</source_csv_delimiter>").append("\r\n");
                sb.append("<source_csv_header>").append(_sourceCsvHeader).append("</source_csv_header>").append("\r\n");
                sb.append("<source_csv_charset>").append(_sourceCsvCharset).append("</source_csv_charset>").append("\r\n");
                sb.append("<source_csv_quote>").append(_sourceCsvQuote).append("</source_csv_quote>").append("\r\n");
            }

            if ("FTP".equals(st)) {
                sb.append("<source_ftp_connection>").append(_dbConnection).append("</source_ftp_connection>").append("\r\n");
                sb.append("<source_ftp_dir>").append(_sourceFtpDir).append("</source_ftp_dir>").append("\r\n");
                sb.append("<source_ftp_file>").append(_sourceFtpFile).append("</source_ftp_file>").append("\r\n");
                sb.append("<source_skip_ftp_file>").append(_sourceSkipFtpFile).append("</source_skip_ftp_file>").append("\r\n");
                sb.append("<source_ftp_load_date>").append(_sourceFtpLoadDate).append("</source_ftp_load_date>").append("\r\n");
            }

            if ("KUDU".equals(tt) || "HIVE".equals(tt)) {
                sb.append("<source_hdfs_path>").append(this.getSourceHdfsDir(_sourceHdfsPath, _sourceHdfsFile)).append("</source_hdfs_path>").append("\r\n");
                sb.append("<source_columns>").append(source_columns_).append("</source_columns>").append("\r\n");
            }
            sb.append("<source_sql>").append(source_sql_).append("</source_sql>").append("\r\n");

            if ("KUDU".equals(tt)) {
                target_table_ = "impala::" + _targetDbName_ + "." + _targetTable;
            }

            if (isBigData2ftp) {
                sb.append("<source_download_file>").append(_targetFtpFile).append("</source_download_file>").append("\r\n");
                sb.append("<source_database>").append(_sourceDbName_).append("</source_database>").append("\r\n");
                sb.append("<target_insert_merge_overwrite>").append("overwrite").append("</target_insert_merge_overwrite>").append("\r\n");
                sb.append("<target_ftp_connection>").append(_targetDbConnection).append("</target_ftp_connection>").append("\r\n");
                sb.append("<target_ftp_dir>").append(_targetFtpDir).append("</target_ftp_dir>").append("\r\n");
            } else {
                sb.append("<target_insert_merge_overwrite>").append(_targetInsertMergeOverwrite).append("</target_insert_merge_overwrite>").append("\r\n");
                sb.append("<target_db_connection>").append(_targetDbConnection).append("</target_db_connection>").append("\r\n");
                sb.append("<target_database>").append(_targetDbName_).append("</target_database>").append("\r\n");
                sb.append("<target_table>").append(target_table_).append("</target_table>").append("\r\n");
                sb.append("<target_primary_columns>").append(target_primary_columns_).append("</target_primary_columns>").append("\r\n");
                sb.append("<target_table_creation>").append(target_table_creation_).append("</target_table_creation>").append("\r\n");
                if (!isBigData2jdbc) {
                    sb.append("<target_columns>").append(target_columns_).append("</target_columns>").append("\r\n");
                    sb.append("<target_partition_columns>").append(target_partition_columns_).append("</target_partition_columns>").append("\r\n");
                }
            }

            sb.append("<ft_column>").append(_ftColumn).append("</ft_column>").append("\r\n");
            sb.append("<ft_count>").append(_ftCount).append("</ft_count>").append("\r\n");
            sb.append("<separate_max>").append(_separateMax).append("</separate_max>").append("\r\n");
            sb.append("<separate_min>").append(_separateMin).append("</separate_min>").append("\r\n");
            sb.append("<syncRate>").append(_syncRate).append("</syncRate>").append("\r\n");
            sb.append("<maxConcurrency>").append(_maxConcurrency).append("</maxConcurrency>").append("\r\n");
            sb.append("<preImportStatement>").append(_preImportStatement).append("</preImportStatement>").append("\r\n");
            sb.append("<postImportStatement>").append(_postImportStatement).append("</postImportStatement>").append("\r\n");
            sb.append("<errorLimitRecord>").append(_errorLimitRecord).append("</errorLimitRecord>").append("\r\n");

            sb.append("<executor_memory>").append(_executorMemory).append("</executor_memory>").append("\r\n");
            sb.append("<executor_cores>").append(_executorCores).append("</executor_cores>").append("\r\n");
            sb.append("<total_executor_cores>").append(_totalExecutorCores).append("</total_executor_cores>").append("\r\n");
            sb.append("<day_by_day>").append(_dayByDay).append("</day_by_day>").append("\r\n");
            sb.append("<clean_rules>").append(clean_rules_).append("</clean_rules>").append("\r\n");
            sb.append("<check_rules>").append(check_rules_).append("</check_rules>").append("\r\n");

            sb.append("</etlprocess>").append("\r\n");
        }
        return sb.toString();
    }

    /*
     *根据项目id和数据源名称 查询数据信息
     * */
    private DmpSyncingDatasource getDmpSyncingDatasource(Integer projectId, String dsName) throws Exception {
        DmpSyncingDatasource ds = new DmpSyncingDatasource();
        ds.setDataStatus("1");
        ds.setDatasourceName(dsName);
        ds.setProjectId(projectId);
        List<DmpSyncingDatasource> dsList = dmpSyncingDatasourceService.findListByParams(ds);
        if (dsList != null && dsList.size() > 0) {
            return dsList.get(0);
        } else {
            return null;
        }
    }

    /*
     * 获取文件类型
     * */
    private String getFileType(String ft, String _sourceHdfsFile, String _sourceFtpFile) {
        if (StringUtils.isBlank(ft)) {
            if (StringUtils.isNotBlank(_sourceHdfsFile)) {
                int idx = _sourceHdfsFile.lastIndexOf(".");
                if (idx > -1) {
                    String nft = _sourceHdfsFile.substring(idx + 1, _sourceHdfsFile.length());
                    if (StringUtils.isNotBlank(nft)) {
                        if ("csv".equalsIgnoreCase(nft)) {
                            ft = "csv";
                        } else if ("json".equalsIgnoreCase(nft)) {
                            ft = "json";
                        } else if ("txt".equalsIgnoreCase(nft)) {
                            ft = "csv";
                        } else {
                            ft = "csv";
                        }
                    }
                }
            }
            if (StringUtils.isNotBlank(_sourceFtpFile)) {
                int idx = _sourceFtpFile.lastIndexOf(".");
                if (idx > -1) {
                    String nft = _sourceFtpFile.substring(idx + 1, _sourceFtpFile.length());
                    if (StringUtils.isNotBlank(nft)) {
                        if ("csv".equalsIgnoreCase(nft)) {
                            ft = "csv";
                        } else if ("json".equalsIgnoreCase(nft)) {
                            ft = "json";
                        } else if ("txt".equalsIgnoreCase(nft)) {
                            ft = "csv";
                        } else {
                            ft = "csv";
                        }
                    }
                }
            }
        }
        return ft;
    }

    private String getTargetDBname(DmpSyncingDatasource ds) {
        String url = ds.getJdbcUrl();
        if (StringUtils.isNotBlank(url)) {
            if (ds.getDatasourceType() == 2) {//sqlserver
                String[] arr1 = url.split("=");
                String[] arr2 = arr1[1].split(";");
                System.out.println(arr2[0]);
                return arr2[0];
            } else {
                String[] arr1 = url.split("/");
                String ls = arr1[arr1.length - 1];
                String[] arr2 = ls.split("\\?");
                return arr2[0];
            }

        } else {
            if (ds.getDatasourceType().equals("11")) {//ftp
                return ds.getDbName();
            }
            return "";
        }
    }

    private String getEtlProcessType(DmpSyncingDatasourceType s, DmpSyncingDatasourceType t, String ft) {
        String sT = s.getDatasourceType().toLowerCase();
        String tT = t.getDatasourceType().toLowerCase();
        String _ft = "";
        if (StringUtils.isNotBlank(ft)) {
            _ft = ft;
        }
        if (s.getDatasourceCatecode().equals("RDBMS")) {
            sT = "jdbc";
        }
        return sT + _ft + "2" + tT;
    }

    private String getTypeSize(String type) {
        if (StringUtils.isNull(type))
            return "";
        String str = "";
        if (type.toUpperCase().indexOf("VARCHAR") != -1) {
            str += "(255)";
        } else if (type.toUpperCase().indexOf("CHAR") != -1) {
            str += "(5)";
        } else {
            // 其他类型
        }
        return str;
    }

    private String[] genSourceColumnsInfo(DmpSyncingDatasourceType s, DmpSyncingDatasourceType t, String ft,
                                          String sTable, String sTable_, String _extract, String extractExpression, List<Map<String, Object>> rColumns,
                                          String sourceDbName) {
        String source_columns = "";
        String source_columns_hive_partition = "";
        String source_table = "<![CDATA[ (select ";
        String source_sql = "<![CDATA[ select ";
        String source_sql_hive_partition = "";
        String clean_rules = "";
        String check_rules = "";
        Map<String, String> relatedTypeMap = getRelatedTypeMap(s, t);
        int i = 0;
        for (Map<String, Object> m : rColumns) {

            String name = (String) m.get("name");
            String type = StringUtils.getStringValue(m.get("type"), "");//relatedTypeMap.get(StringUtils.getStringValue(m.get("type"), ""));
            String isPt = (String) m.get("isPt");
            if ("SQLSERVER".equals(s.getDatasourceType())) { //对sqlserver 修改
                if ("STRING".equals(type)) {
                    source_table += "REPLACE(REPLACE(REPLACE(" + name + ", CHAR(13), '<br1>'),CHAR(10),'<br2>'),',','<comma>') AS " + name;
                } else {
                    source_table += name;
                }
            } else {
//            	source_table += "replace(cast(" + name + " as char), char(10), '') as " + name;
                source_table += name;
            }
            if ("HIVE".equals(t.getDatasourceType()) && "1".equals(isPt)) {
                source_columns_hive_partition += name;
                source_sql_hive_partition += "cast(" + name + " as " + type + ") as ";
                if ("json".equals(ft)) {
                    source_sql_hive_partition += name.toLowerCase();
                } else {
                    source_sql_hive_partition += name;
                }
            } else {
                source_columns += name;
                if ("varchar".equalsIgnoreCase(type)) {
                    source_sql += "cast(" + name + " as string) as ";
                } else if ("char".equalsIgnoreCase(type)) {
                    source_sql += "cast(" + name + " as string) as ";
                } else if ("datetime".equalsIgnoreCase(type)) {
                    source_sql += "cast(" + name + " as timestamp) as ";
                } else {
                    source_sql += "cast(" + name + " as " + type + ") as ";
                }
                if ("json".equals(ft)) {
                    source_sql += name.toLowerCase();
                } else {
                    source_sql += name;
                }
            }
            source_table += ", ";
            if ("HIVE".equals(t.getDatasourceType()) && "1".equals(isPt)) {
                source_columns_hive_partition += ", ";
                source_sql_hive_partition += ", ";
            } else {
                source_columns += ", ";
                source_sql += ", ";
            }
            if (i == rColumns.size() - 1) {
                String loweredStr = source_sql.toLowerCase();
                boolean appjz = loweredStr.indexOf("etl_create_time,") == -1;
                boolean appUpd = loweredStr.indexOf("etl_update_time,") == -1;
                if (appjz) source_sql += "cast(current_timestamp() as string) as etl_create_time, ";
                if (appUpd) source_sql += "cast(current_timestamp() as string) as etl_update_time, ";
            }
            i++;
        }
        if ("HIVE".equals(t.getDatasourceType()) && StringUtils.isNotBlank(source_columns_hive_partition)) {
            //source_sql_hive_partition = source_sql_hive_partition.substring(0, source_sql_hive_partition.length()-2);
            //source_columns_hive_partition = source_columns_hive_partition.substring(0, source_columns_hive_partition.length()-2);
            source_sql += source_sql_hive_partition;
            source_columns += source_columns_hive_partition;
        }
        source_table = source_table.substring(0, source_table.length() - 2);
        if ("RDBMS".equals(s.getDatasourceCatecode())) {
            source_table += " from " + sourceDbName + "." + sTable_;
        } else {
            source_table += " from " + sTable;
        }
        if (!"full".equalsIgnoreCase(_extract) && StringUtils.isNotBlank(extractExpression)) {
            source_table += " " + extractExpression;
        }
        source_table += ") as t]]>";
        if ("FTP".equals(s.getDatasourceType()) || "HDFS".equals(s.getDatasourceType())) {
            sTable = "$$tmpTableName";
        }
        source_sql = source_sql.substring(0, source_sql.length() - 2);
        source_columns = source_columns.substring(0, source_columns.length() - 2);
        source_sql += " from " + sTable + "]]> ";
        return new String[]{source_table, source_columns, source_sql, clean_rules, check_rules};
    }

    private Map<String, String> getRelatedTypeMap(DmpSyncingDatasourceType sds, DmpSyncingDatasourceType tds) {
        /* 封装cast类型，直接根据key=源字段名获取；默认封装2kudu，除非源或目标库是hive库 */
        Map<String, Object> params = new HashMap();
        Map<String, String> typeMap = new HashMap();
        DmpSyncingDatasourceType entity = new DmpSyncingDatasourceType();
        if ("HIVE".equalsIgnoreCase(sds.getDatasourceType()) || "HIVE".equalsIgnoreCase(tds.getDatasourceType())) {
            entity.setDatasourceType("HIVE");
        } else { // 剩余情况均可认定为2kudu
            entity.setDatasourceType("KUDU");
        }

        /*params.put("sourceDsId", sds.getId());
        params.put("targetDsId", dmpSyncingDatasourceTypeService.findList(entity).get(0).getId());
        List<Map<String, Object>> dataTypeMapping = dmpTableFieldSchemaService.getDataTypeMapping(params);
        for (Map<String, Object> mapp : dataTypeMapping) {
            typeMap.put(StringUtils.toString(mapp.get("sourceType")),
                    StringUtils.getStringValue(mapp.get("targetType"), "STRING"));
        }*/
        return typeMap;
    }

    private String getSourceHdfsDir(String filePath, String fileName) {
        if (StringUtils.isNotBlank(fileName)) {
            boolean f1 = filePath.endsWith("/");
            boolean f2 = fileName.startsWith("/");
            if (f1 && f2) {
                fileName = fileName.substring(1, fileName.length());
            }
            if (!f1 && !f2) {
                filePath += "/";
            }
            return filePath + fileName;
        } else {
            return filePath;
        }
    }

    private String[] genTargetColumnsInfo(DmpSyncingDatasourceType s, DmpSyncingDatasourceType t, String tTable,
                                          String _targetBucketCounts, List<Map<String, Object>> wColumns) {
        String target_columns = "";
        String target_table_creation = "create table if not exists " + tTable + "(";
        String target_primary_columns = "";
        String target_partition_columns = "";
        String target_partition_columns_hive = "";
        String target_columns_hive_partition = "";
        String clean_rules = "";
        String check_rules = "";
        int i = 0;
        for (Map<String, Object> m : wColumns) {
            String name = (String) m.get("name");
            String type = (String) m.get("type");
            String comment = m.get("comment") == null ? "" : (String) m.get("comment");
            String isPk = (String) m.get("isPk");
            String isPt = (String) m.get("isPt");
            if ("HIVE".equals(t.getDatasourceType()) && "1".equals(isPt)) {
                target_columns_hive_partition += name;
            } else {
                target_columns += name;
            }

            if (!"1".equals(isPt) && "HIVE".equals(t.getDatasourceType())) {
                target_table_creation += name + " " + type;
                target_table_creation += " comment '" + comment + "'";
            } else if ("KUDU".equals(t.getDatasourceType())) {
                target_table_creation += name + " " + type;
                target_table_creation += "1".equals(isPk) ? " not null " : " null ";
                target_table_creation += " comment '" + comment + "'";
            } else if ("RDBMS".equals(t.getDatasourceCatecode())) {
                // POSTGRESQL支持不指定varchar的长度
                target_table_creation += name + " " + type + getTypeSize(type);
                target_table_creation += "1".equals(isPk) ? " not null " : " null ";
                target_table_creation += "comment '" + comment + "'";
            }
            if ("1".equals(isPk)) {
                target_primary_columns += name + ", ";
            }
            if ("1".equals(isPt)) {
                target_partition_columns += name + ", ";
                target_partition_columns_hive += name + " " + type + ", ";
            }
            String[] rls = this.appendRules(m.get("rules"), name, clean_rules, check_rules);//便利清洗规则
            clean_rules = rls[0];
            check_rules = rls[1];
            if ("HIVE".equals(t.getDatasourceType()) && "1".equals(isPt)) {
                target_columns_hive_partition += ", ";
            } else {
                target_columns += ", ";
            }
            if (!"1".equals(isPt) && "HIVE".equals(t.getDatasourceType())) {
                target_table_creation += ", ";
            } else if ("KUDU".equals(t.getDatasourceType())) {
                target_table_creation += ", ";
            } else if ("RDBMS".equals(t.getDatasourceCatecode())) {
                target_table_creation += ", ";
            }
            if (i == wColumns.size() - 1) {
                // 若源表自带etl_create_time、etl_update_time字段，则不进行追加
                String loweredStr = target_columns.toLowerCase();
                boolean appjz = loweredStr.indexOf("etl_create_time,") == -1;
                boolean appUpd = loweredStr.indexOf("etl_update_time,") == -1;
                if ("KUDU".equals(t.getDatasourceType())) {
                    if (appjz) target_table_creation += "etl_create_time string null comment 'etl_create_time', ";
                    if (appUpd) target_table_creation += "etl_update_time string null comment 'etl_update_time', ";
                } else if ("HIVE".equals(t.getDatasourceType())) {
                    if (appjz) target_table_creation += "etl_create_time string comment 'etl_create_time', ";
                    if (appUpd) target_table_creation += "etl_update_time string comment 'etl_update_time', ";
                } else if ("RDBMS".equals(t.getDatasourceCatecode())) {
                    // 未兼容常用关系型Microsoft Access, DM
                    if ("ORACLE".equals(t.getDatasourceType())) {
                        if (appjz) target_table_creation += "etl_create_time varchar2(30) comment 'etl_create_time', ";
                        if (appUpd) target_table_creation += "etl_update_time varchar2(30) comment 'etl_update_time', ";
                    } else {
                        if (appjz) target_table_creation += "etl_create_time varchar(30) comment 'etl_create_time', ";
                        if (appUpd) target_table_creation += "etl_update_time varchar(30) comment 'etl_update_time', ";
                    }
                }
                if (appjz) target_columns += "etl_create_time, ";
                if (appUpd) target_columns += "etl_update_time, ";

                if ("HIVE".equals(t.getDatasourceType()) && StringUtils.isNotBlank(target_columns_hive_partition)) {
                    target_columns += target_columns_hive_partition;
                }

                if (StringUtils.isNotBlank(target_primary_columns))
                    target_primary_columns = target_primary_columns.substring(0, target_primary_columns.length() - 2);
                if (StringUtils.isNotBlank(target_partition_columns_hive))
                    target_partition_columns_hive = target_partition_columns_hive.substring(0, target_partition_columns_hive.length() - 2);
                if (StringUtils.isNotBlank(target_partition_columns))
                    target_partition_columns = target_partition_columns.substring(0, target_partition_columns.length() - 2);
                if (StringUtils.isNotBlank(target_table_creation))
                    target_table_creation = target_table_creation.substring(0, target_table_creation.length() - 2);
                if (StringUtils.isNotBlank(target_primary_columns)) {
                    if ("KUDU".equals(t.getDatasourceType()) || "RDBMS".equals(t.getDatasourceCatecode())) {
                        target_table_creation += ", primary key (" + target_primary_columns + ") ";
                    }
                }
            }
            i++;
        }
        target_table_creation += ") ";
        if ("KUDU".equals(t.getDatasourceType())) {
            if (StringUtils.isNotBlank(target_partition_columns)) {
                target_table_creation += "partition by hash (" + target_partition_columns + ") ";
            } else {
                if (StringUtils.isNotBlank(target_primary_columns)) {
                    target_table_creation += "partition by hash (" + target_primary_columns + ") ";
                }
            }
            if (StringUtils.isNotBlank(_targetBucketCounts)) {
                target_table_creation += "partitions  " + _targetBucketCounts;
            }
            target_table_creation += " stored as kudu ";
        } else if ("HIVE".equals(t.getDatasourceType())) {
            if (StringUtils.isNotBlank(target_partition_columns)) {
                target_table_creation += "partitioned by (" + target_partition_columns_hive + ") ";
            }
        } else if ("RDBMS".equals(t.getDatasourceCatecode())) {
            // 分区，暂忽略关系型数据库的分区功能
        }
        target_columns = target_columns.substring(0, target_columns.length() - 2);
        return new String[]{target_columns, target_primary_columns, target_partition_columns,
                target_table_creation, clean_rules, check_rules};
    }

    private String[] appendRules(Object obj, String columnName, String clean_rules, String check_rules) {
        List<Map<String, Object>> rules = (List<Map<String, Object>>) obj;
        if (rules.size() > 0 && rules != null) {
            for (Map<String, Object> m : rules) {
                String method = (String) m.get("method");        // 规则方法
                String type = (String) m.get("type");            // 1:清洗规则;0:校验规则
                List<String> params = (ArrayList<String>) m.get("params");    // 规则参数列表，字符串
                String sub_rules = "\t<column_name name=\"" + columnName + "\">\r\n";
                sub_rules += "\t\t<rules_type method=\"" + method + "\">\r\n";
                if (params != null && params.size() > 0)
                    for (String rv : params) {
                        sub_rules += "\t\t\t<rules_value>" + rv + "</rules_value>\r\n";
                    }
                sub_rules += "\t\t</rules_type>\r\n";
                sub_rules += "\t</column_name>\r\n";
                if ("0".equals(type)) {
                    if (StringUtils.isBlank(check_rules))
                        sub_rules = "\r\n" + sub_rules;
                    check_rules += sub_rules;
                }
                if ("1".equals(type)) {
                    if (StringUtils.isBlank(clean_rules))
                        sub_rules = "\r\n" + sub_rules;
                    clean_rules += sub_rules;
                }
            }
        }
        return new String[]{clean_rules, check_rules};
    }
}